program mybipoisson, properties(swml svyj svyr swml)
        version 18
        if replay() {
                if (`"`e(cmd)'"' != "mybipoisson") error 301
                Replay `0'
        }
        else    Estimate `0'
end


program Estimate, eclass sortpreserve
        syntax anything(id="equations" equalok) ///
                [if] [in]                       ///
                [fweight pweight iweight] [,    ///
                noLOg                           /// -ml model- options
                noLRTEST                        ///
                vce(passthru)                   ///
                TRACE                           ///
                *                               ///
        ]
        
        // Check syntax
        mlopts mlopts other, `options'
        // Parse Replay options
        _get_diopts diopts, `other'
        
        // Parse equations
        ParseEqns `anything' 
        foreach j in 1 2 {
                local yvar`j'     `s(yvar`j')'
                local xvars`j'    `s(xvars`j')'
                if "`s(exposure`j')'" != "" {
                        local expvar`j' `s(exposure`j')'
                        local expopt`j' exposure(`expvar`j'')
                }
                if "`s(offset`j')'" != "" {
                        local offvar`j' `s(offset`j')'
                        local offopt`j' offset(`offvar`j'')
                }
                if "`s(noconst`j')'" != "" {
                        local nocon`j'  noconstant
                }
        }
        if "`weight'" != "" {
                local wgt "[`weight'`exp']"
        }
        if "`log'" != "" {
                local qui quietly
        }

        // Mark the estimation sample
        marksample touse
        markout `touse' `yvar1' `xvars1' `exposure1' `offset1' 
        markout `touse' `yvar2' `xvars2' `exposure2' `offset2' 
        
        // Check LHS variables to be nonnegative counts
        CheckLHS `yvar1' `yvar2' `touse'

        _vce_parse `touse', opt(Robust oim opg) argopt(CLuster): `wgt', `vce'

        // Initial values
        if "`trace'" != "" {
                local noise noisily
        }
        foreach j in 1 2 {
                tempname b0`j'
                capture `noise' poisson                                 ///
                        `yvar`j'' `xvars`j'' `wgt' if `touse' ,         ///
                        `offopt`j'' `expopt`j'' `nocon`j''
                if _rc {
                        di in smcl as error                             ///
                           "Could not find initial values for " _c      ///
                           "equation `j'."
                        exit _rc
                }
                mat `b0`j'' = e(b)
                mat coleq `b0`j'' = "`yvar`j'':"  
        }
        tempname b0
        mat `b0' = `b01', `b02', 0  
        local initopt init(`b0', copy)

        local diparm1 diparm(ln_xi0, exp label("xi0"))

        // fit the full model
        ml model lf2 mybipoisson_lf2()                     /// 
                (`yvar1' : `yvar1' = `xvars1',             ///
                          `nocon1' `offopt1' `expopt1'     ///
                )                                          ///
                (`yvar2' : `yvar2' = `xvars2',             ///
                        `nocon2' `offopt2' `expopt2'       ///
                )                                          ///
                /ln_xi0                                    ///
                `wgt' if `touse',                          ///
                `vce'                                      ///
                `log'                                      ///
                `mlopts'                                   ///
                `diparm'                                   ///
                `initopt'                                  ///
                missing                                    ///
                maximize

        ereturn scalar k_aux    = 1
        ereturn local  predict  mybipoisson_p
        ereturn local  cmd      mybipoisson

        Replay , `diopts'
end

program Replay
        syntax [, *]
        ml display , `options'
end

//      ParseEqns anything
//      Modified from mysureg.ado to fit our needs.
//
//      Takes "anything" and parses as 
//              (y1 xvars1 [, EXPosure(varname) OFFset(varname) noCONStant])
//              (y2 xvars2 [, EXPosure(varname) OFFset(varname) noCONStant])
//
//      Returns, for j in {1,2} 
//              s(yvar`j')
//              s(xvars`j')
//              s(exposure`j')
//              s(offset`j')
//              s(noconst`j')
//
program ParseEqns, sclass
        gettoken eq eqns : 0 , match(paren) parse("()") bind
        local p 0
        while "`eq'" != "" {
                local ++p
                local 0 `eq'
                syntax varlist(fv) [, EXPosure(varname) OFFset(varname) ///
                                      noCONStant ]
                gettoken lhs rhs : varlist
                local yvar`p'      `lhs'
                local xvars`p'     `rhs'
                local exposure`p'  `exposure' 
                local offset`p'    `exposure' 
                local noconst`p'   `constant'
                gettoken eq eqns : eqns , match(paren) parse("()") bind
        }
        if `p' != 2 {
                di in smcl as error "You must specify two, and only two, " _c   
                di in smcl as error "equations."
                exit 198
        }

        sreturn clear
        foreach p in 1 2 {
                sreturn local yvar`p'     `yvar`p''
                sreturn local xvars`p'    `xvars`p''
                sreturn local exposure`p' `exposure`p''
                sreturn local offset`p'   `offset`p''
                sreturn local noconst`p'  `nconst`p''
        }

end

program CheckLHS

        args yvar1 yvar2 touse

        local isae in smcl as error
        foreach j in 1 2 {
                summ `yvar`j'' if `touse', meanonly
                if r(min) < 0 {
                        di `isae' "`yvar`j'' cannot be negative."
                        exit 398
                }
                if r(max) == r(min) {
                        di `isae' "`yvar`j'' has no variance."
                }
                capture assert `yvar`j'' == floor(`yvar`j'')
                if _rc {
                        di `isae' "`yvar`j'' has non-integer values."
                        exit 398
                }
        }

end


exit

